import 'package:flutter/foundation.dart';
import 'package:flutter_flex_cart/src/detail.dart';
import 'package:flutter_flex_cart/src/item.dart';

/// Core Cart Base class
/// This class help to keep the state of the cart.
/// If you want to add customer details use billable with cart.
/// * [D] CoreDetail<I,ID>
/// * [ITEM] CoreItem<ID>
/// * [ID] Item ID type
abstract class CoreCartBase<
    D extends CoreDetail<ITEM, ID>,
    ITEM extends CoreItem<ID>,
    ID> with ChangeNotifier, DiagnosticableTreeMixin {
  /// List of cart product details.
  /// * [D] CoreDetail<I,ID>
  final List<D> _details = [];

  /// Iterable of cart product details.
  /// * [D] CoreDetail<I,ID>
  Iterable<D> get details => List.unmodifiable(_details);

  int get noOfItems => _details.length;

  double get total => _details.fold(
        0,
        (sum, next) => sum + (next.lineTotal),
      );

  /// override this method to increase qty if there is already product detail available.
  void addDetail(D detail) {
    _details.add(detail);
    notifyListeners();
  }

  void removeDetail(int index) {
    _details.removeAt(index);
    notifyListeners();
  }

  bool isInTheCart(ID id) =>
      _details.indexWhere((element) => element.item.id == id) != -1;

  void reset() {
    _details.clear();
    notifyListeners();
  }

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties.add(
      IntProperty('noOfItems', noOfItems),
    );
    properties.add(
      DoubleProperty('total', total),
    );
    properties.add(
      IterableProperty<D>(
        'details',
        _details,
        style: DiagnosticsTreeStyle.dense,
      ),
    );
  }
}
